==> core-2.1.12-sources.jar: com/sun/enterprise/module/bootstrap/ContextDuplicatePostProcessor.java <==
/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2012 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package com.sun.enterprise.module.bootstrap;

import java.util.HashSet;
import java.util.Set;

import org.glassfish.hk2.api.Context;
import org.glassfish.hk2.api.PerLookup;
import org.glassfish.hk2.bootstrap.PopulatorPostProcessor;
import org.glassfish.hk2.runlevel.RunLevel;
import org.glassfish.hk2.utilities.DescriptorImpl;

/**
 * This class will weed out and duplicate implementations of Context and RunLevel, which can
 * happen if the same jar is on the classpath twice.  This service must be put into the
 * ServiceLocator prior to populating from the inhabitant files, and hence is not
 * annotated with &#64;Service
 * 
 * @author jwells
 *
 */
@PerLookup
public class ContextDuplicatePostProcessor implements PopulatorPostProcessor {
    private final HashSet<DescriptorImpl> dupSet = new HashSet<DescriptorImpl>();

    /* (non-Javadoc)
     * @see org.glassfish.hk2.bootstrap.PopulatorPostProcessor#process(org.glassfish.hk2.utilities.DescriptorImpl)
     */
    @Override
    public DescriptorImpl process(DescriptorImpl descriptorImpl) {
        if (dupSet.contains(descriptorImpl)) {
            return null;
        }
        dupSet.add(descriptorImpl);
        
        return descriptorImpl;
    }

}

==> core-2.1.15-sources.jar: com/sun/enterprise/module/bootstrap/ContextDuplicatePostProcessor.java <==
/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2012 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package com.sun.enterprise.module.bootstrap;

import java.util.HashSet;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.inject.Inject;

import org.glassfish.hk2.api.ActiveDescriptor;
import org.glassfish.hk2.api.PerLookup;
import org.glassfish.hk2.api.ServiceLocator;
import org.glassfish.hk2.bootstrap.PopulatorPostProcessor;
import org.glassfish.hk2.utilities.BuilderHelper;
import org.glassfish.hk2.utilities.DescriptorImpl;

/**
 * This class will weed out and duplicate implementations of anything populated into the
 * locator.  This service must be put into the ServiceLocator prior to populating from the
 * inhabitant files, and hence is not annotated with &#64;Service
 * 
 * @author jwells
 *
 */
@PerLookup
public class ContextDuplicatePostProcessor implements PopulatorPostProcessor {
    private final HashSet<DescriptorImpl> dupSet = new HashSet<DescriptorImpl>();
    
    @Inject
    private ServiceLocator locator;

    /* (non-Javadoc)
     * @see org.glassfish.hk2.bootstrap.PopulatorPostProcessor#process(org.glassfish.hk2.utilities.DescriptorImpl)
     */
    @Override
    public DescriptorImpl process(DescriptorImpl descriptorImpl) {
        if (dupSet.contains(descriptorImpl)) {
            return null;
        }
        dupSet.add(descriptorImpl);
        
        return descriptorImpl;
    }
    
    /**
     * This method initializes the dupSet with all of the existing descriptors
     * in the system, so that the process method will not add any duplicates
     */
    @SuppressWarnings("unused")
    @PostConstruct
    private void postConstruct() {
        List<ActiveDescriptor<?>> allDescriptors =
                locator.getDescriptors(BuilderHelper.allFilter());
        
        for (ActiveDescriptor<?> aDescriptor : allDescriptors) {
            dupSet.add(new DescriptorImpl(aDescriptor));
        }
    }

}
